@@ -3,13 +3,13 @@ from collections import defaultdict
 from queue import Queue, Empty
 from typing import Any, Type, Union
 
-from tsuchinoko.assets import path
-
 try:
     from yaml import CLoader as Loader, CDumper as Dumper, dump, load
 except ImportError:
     from yaml import Loader, Dumper
 
+import zmq
+from zmq.error import ZMQError
 import numpy as np
 from PySide2.QtGui import QIcon
 from loguru import logger
@@ -17,8 +17,8 @@ from pyqtgraph import mkBrush, mkPen, HistogramLUTWidget, PlotItem
 from pyqtgraph.dockarea import DockArea
 from qtmodern.styles import dark
 from qtpy.QtWidgets import QMainWindow, QApplication, QHBoxLayout, QWidget, QMenuBar, QAction, QStyle, QFileDialog, QDialog, QMessageBox
-from zmq.error import ZMQError
 
+from tsuchinoko.assets import path
 from tsuchinoko.adaptive import Data
 from tsuchinoko.core import CoreState
 from tsuchinoko.core.messages import PauseRequest, StartRequest, GetParametersRequest, SetParameterRequest, PartialDataRequest, FullDataRequest, StopRequest, Message, StateRequest, StateResponse, GetParametersResponse, FullDataResponse, PartialDataResponse, MeasureRequest, \
@@ -78,6 +78,8 @@ class MainWindow(QMainWindow):
 
         dark(QApplication.instance())
 
+        self.context = zmq.Context()
+        self.socket = None
         self.core_address = core_address
         self.init_socket()
 
@@ -88,7 +90,7 @@ class MainWindow(QMainWindow):
         self.configuration_widget.sigRequestParameters.connect(self.request_parameters)
         request_relay.sigRequestMeasure.connect(self.request_measure)
 
-        self.update_thread = QThreadFutureIterator(self.update)
+        self.update_thread = QThreadFutureIterator(self.update, finished_slot=self.close_zmq, name='tsuchinoko-update')
         self.update_thread.start()
 
         self.data: Data = Data()
@@ -104,12 +106,14 @@ class MainWindow(QMainWindow):
         self.subscribe(self.log_widget.log_exception, ExceptionResponse)
 
     def init_socket(self):
-        import zmq
-        context = zmq.Context()
+        if self.socket:
+            logger.debug("Closing socket")
+            self.socket.close()
 
         #  Socket to talk to server
         logger.info("Connecting to core server…")
-        self.socket = context.socket(zmq.REQ)
+        self.socket = self.context.socket(zmq.REQ)
+        self.socket.setsockopt(zmq.LINGER, 5)
         self.socket.connect(f"tcp://{self.core_address}:5555")
         self.socket.RCVTIMEO = 5000
         self.message_queue = Queue()
@@ -144,6 +148,7 @@ class MainWindow(QMainWindow):
         self.last_data_size = 0
 
         while True:
+            yield
             request = None
 
             if self.state_manager_widget.state == CoreState.Stopping:
@@ -178,7 +183,8 @@ class MainWindow(QMainWindow):
                 logger.warning(f'Unable to connect to core server at {self.core_address}...')
                 time.sleep(1)
                 # logger.exception(ex)
-                self.init_socket()
+                if self.context:
+                    self.init_socket()
                 self.data = Data()  # wipeout data and get a full update next time
                 self.last_data_size = 0
                 self.state_manager_widget.update_state(CoreState.Connecting)
@@ -409,3 +415,35 @@ class MainWindow(QMainWindow):
 
         self.data = Data()
         self.graph_manager_widget.clear()
+
+    def close_zmq(self):
+        if self.update_thread.running:
+            logger.info('waiting for update thread to finish')
+            self.update_thread.requestInterruption()
+            self.update_thread.wait()
+        if self.socket:
+            logger.debug('Closing socket')
+            self.socket.close()
+            self.socket = None
+        if self.context:
+            logger.debug('Closing context')
+            self.context.term()
+            self.context = None
+
+    def closeEvent(self, event):
+        if self.data and len(self.data):
+            result = QMessageBox.question(self,
+                                      'Save data?',
+                                      "You have unsaved data in the active experiment. Do you want to save the data?",
+                                      buttons=QMessageBox.StandardButtons(QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel),
+                                      defaultButton=QMessageBox.Yes)
+            if result == QMessageBox.Yes:
+                self.save_data()
+            if result in [QMessageBox.Yes, QMessageBox.No]:
+                event.accept()
+                self.close_zmq()
+            else:
+                event.ignore()
+        else:
+            event.accept()
+            self.close_zmq()
